@@ -1,6 +1,8 @@
 from .version import __version__, __versiondate__
+from .settings import *
 from .utils import *
 from .defaults import *
+from .parameters import *
 from .sim import *
 from .interventions import *
 from .analyzers import *
