@@ -8,7 +8,9 @@ import numpy as np
 import pylab as pl
 import pandas as pd
 import sciris as sc
-from . import defaults as fpp # CK: TODO: FIX
+from .settings import options as fpo
+from . import defaults as fpd
+from . import parameters as fpp
 from . import sim as fps
 
 
@@ -35,6 +37,7 @@ default_flags = sc.objdict(
     cdr           = 1, # Crude death rate at end of sim in model vs data; 'crude_death_rate'
     cbr           = 1, # Crude birth rate (per 1000 inhabitants); 'crude_birth_rate'
     tfr           = 1, # Total fertility rate
+    asfr          = 1, # Age-specific fertility rate
 )
 
 
@@ -53,8 +56,8 @@ class Experiment(sc.prettyobj):
     def __init__(self, pars=None, flags=None, label=None, **kwargs):
         self.flags = sc.mergedicts(default_flags, flags, _copy=True) # Set flags for what gets run
         self.pars = pars if pars else fpp.pars(**kwargs)
-        self.model_to_calib = sc.objdict()
-        self.dhs_data = sc.objdict()
+        self.model = sc.objdict()
+        self.data = sc.objdict()
         self.method_keys = None
         self.initialized = False
         self.label = label
@@ -85,9 +88,9 @@ class Experiment(sc.prettyobj):
 
         json = self.load_data('basic_dhs')
 
-        self.dhs_data.update(json)
+        self.data.update(json)
 
-        self.dhs_data['pregnancy_parity'] = self.load_data('pregnancy_parity')
+        self.data['pregnancy_parity'] = self.load_data('pregnancy_parity')
 
         # Extract population size over time
         if self.pars:
@@ -95,18 +98,18 @@ class Experiment(sc.prettyobj):
         else:
             n = 1000 # Use default if not available
             print(f'Warning: parameters not defined, using default of n={n}')
-        pop_size = self.load_data('popsize', header=None)  # From World Bank
-        self.dhs_data['pop_years'] = pop_size.iloc[0,:].to_numpy()
-        self.dhs_data['pop_size'] = pop_size.iloc[1,:].to_numpy() / (pop_size.iloc[1,0] / n)  # Corrected for # of agents, needs manual adjustment for # agents
+        pop_size = self.load_data('popsize')
+        self.data['pop_years'] = pop_size.year.to_numpy()
+        self.data['pop_size']  = pop_size.popsize.to_numpy() / (pop_size.popsize[0] / n)  # Corrected for # of agents, needs manual adjustment for # agents
 
         # Extract population growth rate
-        data_growth_rate = self.pop_growth_rate(self.dhs_data['pop_years'], self.dhs_data['pop_size'])
-        self.dhs_data['pop_growth_rate'] = data_growth_rate
+        data_growth_rate = self.pop_growth_rate(self.data['pop_years'], self.data['pop_size'])
+        self.data['pop_growth_rate'] = data_growth_rate
 
         # Extract mcpr over time
-        mcpr = self.load_data('mcpr', header=None)
-        self.dhs_data['mcpr_years'] = mcpr.iloc[:,0].to_numpy()
-        self.dhs_data['mcpr'] = mcpr.iloc[:,1].to_numpy()
+        mcpr = self.load_data('mcpr')
+        self.data['mcpr_years'] = mcpr.iloc[:,0].to_numpy()
+        self.data['mcpr'] = mcpr.iloc[:,1].to_numpy()
 
         self.initialized = True
 
@@ -124,7 +127,7 @@ class Experiment(sc.prettyobj):
         return growth_rate
 
 
-    def run_model(self, pars=None, mother_ids=False):
+    def run_model(self, pars=None, **kwargs):
         ''' Create the sim and run the model '''
 
         if not self.initialized:
@@ -133,7 +136,7 @@ class Experiment(sc.prettyobj):
         if pars is None:
             pars = self.pars
 
-        self.sim = fps.Sim(pars=pars, mother_ids=mother_ids)
+        self.sim = fps.Sim(pars=pars, **kwargs)
         self.sim.run()
         self.post_process_sim()
 
@@ -147,7 +150,7 @@ class Experiment(sc.prettyobj):
         # Store dataframe of agent's age, pregnancy status, and parity
         model_pregnancy_parity = self.sim.store_postpartum()
         model_pregnancy_parity = model_pregnancy_parity.drop(['PP0to5', 'PP6to11', 'PP12to23', 'NonPP'], axis=1)
-        self.model_to_calib['pregnancy_parity'] = model_pregnancy_parity
+        self.model['pregnancy_parity'] = model_pregnancy_parity
         self.method_keys = list(self.sim['methods']['map'].keys())
         return
 
@@ -160,16 +163,17 @@ class Experiment(sc.prettyobj):
         if self.flags.cdr:      self.model_crude_death_rate()
         if self.flags.cbr:      self.model_crude_birth_rate()
         if self.flags.tfr:      self.model_data_tfr()
+        if self.flags.asfr:     self.model_data_asfr()
         return
 
 
     def model_pop_size(self):
 
-        self.model_to_calib['pop_size'] = self.model_results['pop_size']
-        self.model_to_calib['pop_years'] = self.model_results['tfr_years']
+        self.model['pop_size'] = self.model_results['pop_size']
+        self.model['pop_years'] = self.model_results['tfr_years']
 
-        model_growth_rate = self.pop_growth_rate(self.model_to_calib['pop_years'], self.model_to_calib['pop_size'])
-        self.model_to_calib['pop_growth_rate'] = model_growth_rate
+        model_growth_rate = self.pop_growth_rate(self.model['pop_years'], self.model['pop_size'])
+        self.model['pop_growth_rate'] = model_growth_rate
 
         return
 
@@ -179,14 +183,14 @@ class Experiment(sc.prettyobj):
         model = {'years': self.model_results['t'], 'mcpr': self.model_results['mcpr']}
         model_frame = pd.DataFrame(model)
 
-        data_years = self.dhs_data['mcpr_years'].tolist()
-
+        # Filter to matching years
+        data_years = self.data['mcpr_years'].tolist()
         filtered_model = model_frame.loc[model_frame.years.isin(data_years)]
-
         model_mcpr = filtered_model['mcpr'].to_numpy()
+        mcpr_years = filtered_model['years'].to_numpy()
 
-        self.model_to_calib['mcpr'] = model_mcpr*100 # Since data is in 100
-        self.model_to_calib['mcpr_years'] = self.dhs_data['mcpr_years']
+        self.model['mcpr'] = model_mcpr*100 # Since data is in 100
+        self.model['mcpr_years'] = mcpr_years
 
         return
 
@@ -196,52 +200,73 @@ class Experiment(sc.prettyobj):
         Calculate maternal mortality in model over most recent 3 years
         '''
 
-        maternal_deaths = pl.sum(self.model_results['maternal_deaths'][-mpy * 3:])
-        births_last_3_years = pl.sum(self.model_results['births'][-mpy * 3:])
-        self.model_to_calib['maternal_mortality_ratio'] = (maternal_deaths / births_last_3_years) * 100000
+        maternal_deaths = np.sum(self.model_results['maternal_deaths'][-mpy * 3:])
+        births_last_3_years = np.sum(self.model_results['births'][-mpy * 3:])
+        self.model['maternal_mortality_ratio'] = (maternal_deaths / births_last_3_years) * 100000
 
         return
 
 
     def model_infant_mortality_rate(self):
 
-        infant_deaths = pl.sum(self.model_results['infant_deaths'][-mpy:])
-        births_last_year = pl.sum(self.model_results['births'][-mpy:])
-        self.model_to_calib['infant_mortality_rate'] = (infant_deaths / births_last_year) * 1000
+        infant_deaths = np.sum(self.model_results['infant_deaths'][-mpy:])
+        births_last_year = np.sum(self.model_results['births'][-mpy:])
+        self.model['infant_mortality_rate'] = (infant_deaths / births_last_year) * 1000
 
         return
 
 
     def model_crude_death_rate(self):
-        total_deaths = pl.sum(self.model_results['deaths'][-mpy:]) + \
-                       pl.sum(self.model_results['infant_deaths'][-mpy:]) + \
-                       pl.sum(self.model_results['maternal_deaths'][-mpy:])
-        self.model_to_calib['crude_death_rate'] = (total_deaths / self.model_results['pop_size'][-1]) * 1000
+        total_deaths = np.sum(self.model_results['deaths'][-mpy:]) + \
+                       np.sum(self.model_results['infant_deaths'][-mpy:]) + \
+                       np.sum(self.model_results['maternal_deaths'][-mpy:])
+        self.model['crude_death_rate'] = (total_deaths / self.model_results['pop_size'][-1]) * 1000
         return
 
 
     def model_crude_birth_rate(self):
-        births_last_year = pl.sum(self.model_results['births'][-mpy:])
-        self.model_to_calib['crude_birth_rate'] = (births_last_year / self.model_results['pop_size'][-1]) * 1000
+        births_last_year = np.sum(self.model_results['births'][-mpy:])
+        self.model['crude_birth_rate'] = (births_last_year / self.model_results['pop_size'][-1]) * 1000
         return
 
 
     def model_data_tfr(self):
 
         # Extract tfr over time in data - keep here to ignore dhs data if not using tfr for calibration
-        tfr = self.load_data('tfr', header=None)  # From DHS
-        self.dhs_data['tfr_years'] = tfr.iloc[:, 0].to_numpy()
-        self.dhs_data['total_fertility_rate'] = tfr.iloc[:, 1].to_numpy()
+        tfr = self.load_data('tfr')  # From DHS
+        self.data['tfr_years'] = tfr.iloc[:, 0].to_numpy()
+        self.data['total_fertility_rate'] = tfr.iloc[:, 1].to_numpy()
+
+        self.model['tfr_years'] = self.model_results['tfr_years']
+        self.model['total_fertility_rate'] = self.model_results['tfr_rates']
+        return
+
+
+    def model_data_asfr(self, ind=-1):
+
+        # Extract ASFR for different age bins
+        asfr = self.load_data('asfr')  # From DHS
+        self.data['asfr_bins'] = list(asfr.iloc[:, 0])
+        self.data['asfr']      = asfr.iloc[:, 1].to_numpy()
+
+        # Model extraction
+        age_bins = list(fpd.age_bin_map.keys())
+        self.model['asfr_bins'] = age_bins
+        self.model['asfr'] = []
+        for ab in age_bins:
+            val = self.model_results['asfr'][ab][ind] # Only use one index (default: last) CK: TODO: match year automatically
+            self.model['asfr'].append(val)
+
+        # Check
+        assert self.data['asfr_bins'] == self.model['asfr_bins'], f'ASFR data age bins do not match sim: {sc.strjoin(age_bins)}'
 
-        self.model_to_calib['tfr_years'] = self.model_results['tfr_years']
-        self.model_to_calib['total_fertility_rate'] = self.model_results['tfr_rates']
         return
 
 
     def extract_skyscrapers(self):
 
         # Set up
-        min_age = 15
+        min_age = 15 # CK: TODO: remove hardcoding
         max_age = 50
         bin_size = 5
         age_bins = pl.arange(min_age, max_age, bin_size)
@@ -251,11 +276,11 @@ class Experiment(sc.prettyobj):
         x_age = pl.arange(n_age)
 
         # Load data
-        data_parity_bins = pl.arange(0, 18)
-        sky_raw_data = self.load_data('skyscrapers', header=None)
-        sky_raw_data = sky_raw_data[sky_raw_data[0] == year_str]
+        data_parity_bins = pl.arange(0, 18) # CK: TODO: refactor
+        sky_raw_data = self.load_data('skyscrapers')
+        sky_raw_data = sky_raw_data[sky_raw_data.year == year_str]
         # sky_parity = sky_raw_data[2].to_numpy() # Not used currently
-        sky_props = sky_raw_data[3].to_numpy()
+        sky_props = sky_raw_data.percentage.to_numpy()
         sky_arr = sc.odict()
 
         sky_arr['Data'] = pl.zeros((len(age_bins), len(parity_bins)))
@@ -280,8 +305,8 @@ class Experiment(sc.prettyobj):
         for key in ['Data', 'Model']:
             sky_arr[key] /= sky_arr[key].sum() / 100
 
-        self.dhs_data['skyscrapers'] = sky_arr['Data']
-        self.model_to_calib['skyscrapers'] = sky_arr['Model']
+        self.data['skyscrapers'] = sky_arr['Data']
+        self.model['skyscrapers'] = sky_arr['Model']
         self.age_bins = age_bins
         self.parity_bins = parity_bins
 
@@ -315,9 +340,9 @@ class Experiment(sc.prettyobj):
                                           pl.percentile(first, 75)])
 
         # Save to dictionary
-        self.dhs_data['spacing_bins'] = np.array(data_spacing_counts.values())
-        self.dhs_data['spacing_stats'] = data_spacing_stats
-        self.dhs_data['age_first_stats'] = data_age_first_stats
+        self.data['spacing_bins'] = np.array(data_spacing_counts.values())
+        self.data['spacing_stats'] = data_spacing_stats
+        self.data['age_first_stats'] = data_age_first_stats
 
         # From model
         model_age_first = []
@@ -345,16 +370,16 @@ class Experiment(sc.prettyobj):
             model_age_first_stats = np.array([np.percentile(model_age_first, 25),
                                             np.percentile(model_age_first, 50),
                                             np.percentile(model_age_first, 75)])
-        except Exception as E:
+        except Exception as E: # pragma: nocover
             print(f'Could not calculate birth spacing, returning zeros: {E}')
             model_spacing_counts = {k:0 for k in spacing_bins.keys()}
             model_spacing_stats = np.zeros(data_spacing_stats.shape)
             model_age_first_stats = np.zeros(data_age_first_stats.shape)
 
         # Save arrays to dictionary
-        self.model_to_calib['spacing_bins'] = np.array(model_spacing_counts.values())
-        self.model_to_calib['spacing_stats'] = model_spacing_stats
-        self.model_to_calib['age_first_stats'] = model_age_first_stats
+        self.model['spacing_bins'] = np.array(model_spacing_counts.values())
+        self.model['spacing_stats'] = model_spacing_stats
+        self.model['age_first_stats'] = model_age_first_stats
 
         return
 
@@ -411,8 +436,8 @@ class Experiment(sc.prettyobj):
             else:
                 model_labels[d] = ''
 
-        self.dhs_data['method_counts'] = np.array(data_method_counts.values())
-        self.model_to_calib['method_counts'] = np.array(model_method_counts.values())
+        self.data['method_counts'] = np.array(data_method_counts.values())
+        self.model['method_counts'] = np.array(model_method_counts.values())
 
         return
 
@@ -421,21 +446,21 @@ class Experiment(sc.prettyobj):
         index = [0, 1, 2, 3, 7] #indices of count, min, and max to drop from descriptive stats
         # Keep mean [1], 25% [4], 50%[5], 75% [6]
 
-        data = self.dhs_data['pregnancy_parity'] # Copy DataFrame for mainupation
+        data = self.data['pregnancy_parity'] # Copy DataFrame for mainupation
         preg = data[data['Pregnant'] == 1]
         stat = preg['Age'].describe()
         data_stats_all = stat.to_numpy()
         data_stats = np.delete(data_stats_all, index)
 
-        self.dhs_data['age_pregnant_stats'] = data_stats  # Array of mean, std, 25%, 50%, 75% of ages of agents currently pregnant
+        self.data['age_pregnant_stats'] = data_stats  # Array of mean, std, 25%, 50%, 75% of ages of agents currently pregnant
 
-        model = self.model_to_calib['pregnancy_parity']  # Copy DataFrame for manipulation
+        model = self.model['pregnancy_parity']  # Copy DataFrame for manipulation
         pregnant = model[model['Pregnant'] == 1]
         stats = pregnant['Age'].describe()
         model_stats_all = stats.to_numpy()
         model_stats = np.delete(model_stats_all, index)
 
-        self.model_to_calib['age_pregnant_stats'] = model_stats
+        self.model['age_pregnant_stats'] = model_stats
 
         parity_data = data.groupby('Parity')['Age'].describe()
         parity_data = parity_data.head(11) # Include only parities 0-10 to exclude outliers
@@ -443,7 +468,7 @@ class Experiment(sc.prettyobj):
         parity_data.fillna(0)
         parity_data_stats = parity_data.to_numpy()
 
-        self.dhs_data['age_parity_stats'] = parity_data_stats
+        self.data['age_parity_stats'] = parity_data_stats
 
         parity_model = model.groupby('Parity')['Age'].describe()
         parity_model = parity_model.head(11)
@@ -451,13 +476,13 @@ class Experiment(sc.prettyobj):
         parity_model.fillna(0)
         parity_model_stats = parity_model.to_numpy()
 
-        self.model_to_calib['age_parity_stats'] = parity_model_stats
+        self.model['age_parity_stats'] = parity_model_stats
 
 
     def compute_fit(self, *args, **kwargs):
         ''' Compute how good the fit is '''
-        data = sc.dcp(self.dhs_data)
-        sim = sc.dcp(self.model_to_calib)
+        data = sc.dcp(self.data)
+        sim = sc.dcp(self.model)
         for k in data.keys():
             data[k] = sc.promotetoarray(data[k])
             data[k] = data[k].flatten()
@@ -480,8 +505,8 @@ class Experiment(sc.prettyobj):
             del self.people
 
         # Remove raw dataframes of pregnancy / parity data from dictionary
-        del self.dhs_data['pregnancy_parity']
-        del self.model_to_calib['pregnancy_parity']
+        del self.data['pregnancy_parity']
+        del self.model['pregnancy_parity']
 
         # Compute comparison
         self.df = self.compare()
@@ -497,21 +522,21 @@ class Experiment(sc.prettyobj):
         ''' Run the model and post-process the results '''
         self.run_model(pars=pars)
         self.post_process_results(keep_people=keep_people, compute_fit=compute_fit, **kwargs)
-        return
+        return self
 
 
     def compare(self):
         ''' Create and print a comparison between model and data '''
         # Check that keys match
-        data_keys = self.dhs_data.keys()
-        model_keys = self.model_to_calib.keys()
+        data_keys = self.data.keys()
+        model_keys = self.model.keys()
         assert set(data_keys) == set(model_keys), 'Data and model keys do not match'
 
         # Compare the two
         comparison = []
         for key in data_keys:
-            dv = self.dhs_data[key] # dv = "Data value"
-            mv = self.model_to_calib[key] # mv = "Model value"
+            dv = self.data[key] # dv = "Data value"
+            mv = self.model[key] # mv = "Model value"
             cmp = sc.objdict(key=key,
                              d_type=type(dv),
                              m_type=type(mv),
@@ -542,20 +567,21 @@ class Experiment(sc.prettyobj):
         summary.model = sc.objdict()
         summary.data = sc.objdict()
 
-        data = self.dhs_data
-        model = self.model_to_calib
+        data = self.data
+        model = self.model
         keys = model.keys()
 
         # Compare the two
         for key in keys:
-            dv = data[key] # dv = "Data value"
-            mv = model[key] # mv = "Model value"
-            if sc.isnumber(mv) and sc.isnumber(dv):
-                summary.data[key] = dv
-                summary.model[key] = mv
-            else:
-                summary.data[key+'_mean'] = np.mean(dv)
-                summary.model[key+'_mean'] = np.mean(mv)
+            if not (key.endswith('_years') or key.endswith('_bins')):
+                dv = data[key] # dv = "Data value"
+                mv = model[key] # mv = "Model value"
+                if sc.isnumber(mv) and sc.isnumber(dv):
+                    summary.data[key] = dv
+                    summary.model[key] = mv
+                else:
+                    summary.data[key+'_mean'] = np.mean(dv)
+                    summary.model[key+'_mean'] = np.mean(mv)
 
         self.summary = summary
         self.summary_df = pd.DataFrame(summary)
@@ -583,8 +609,8 @@ class Experiment(sc.prettyobj):
 
         **Examples**::
 
-            json = calib.to_json()
-            calib.to_json('results.json')
+            json = exp.to_json()
+            exp.to_json('results.json')
         '''
         d = self.summarize()
         if filename is None:
@@ -595,10 +621,10 @@ class Experiment(sc.prettyobj):
         return output
 
 
-    def plot(self, axes_args=None, do_maximize=True, do_show=True):
+    def plot(self, do_show=None, do_save=None, filename='fp_experiment.png', axis_args=None, do_maximize=True):
         ''' Plot the model against the data '''
-        data = self.dhs_data
-        sim = self.model_to_calib
+        data = self.data
+        sim = self.model
 
         # Set up keys structure and remove non-plotted keys
         keys = ['rates'] + list(data.keys())
@@ -606,183 +632,179 @@ class Experiment(sc.prettyobj):
                      'infant_mortality_rate',
                      'crude_death_rate',
                      'crude_birth_rate']
-        non_calibrated_keys = ['pop_years', 'mcpr_years', 'tfr_years']
+        non_calibrated_keys = ['pop_years', 'mcpr_years', 'tfr_years', 'asfr_bins']
         for key in rate_keys + non_calibrated_keys:
-            keys.remove(key)
+            if key in keys:
+                keys.remove(key)
         nkeys = len(keys)
-        expected = 12
+        expected = 13
         if nkeys != expected:
-            errormsg = f'Number of keys changed -- expected {expected}, actually {nkeys}'
+            errormsg = f'Number of keys changed -- expected {expected}, actually {nkeys} -- did you use run_model() instead of run()?'
             raise ValueError(errormsg)
 
-        fig, axs = pl.subplots(nrows=4, ncols=3)
-        pl.subplots_adjust(**sc.mergedicts(dict(bottom=0.05, top=0.97, left=0.05, right=0.97, wspace=0.3, hspace=0.3), axes_args))
+        with fpo.with_style():
 
-        #%% Do the plotting!
+            fig, axs = pl.subplots(nrows=4, ncols=3)
+            pl.subplots_adjust(**sc.mergedicts(dict(bottom=0.05, top=0.97, left=0.05, right=0.97, wspace=0.3, hspace=0.3), axis_args))
 
-        # Rates
-        ax = axs[0,0]
-        height = 0.4
-        n_rates = len(rate_keys)
-        y = np.arange(n_rates)
-        data_rates = np.array([data[k] for k in rate_keys])
-        sim_rates  = np.array([sim[k] for k in rate_keys])
-        ax.barh(y=y+height/2, width=data_rates, height=height, align='center', label='Data')
-        ax.barh(y=y-height/2, width=sim_rates,  height=height, align='center', label='Sim')
-        ax.set_title('Rates')
-        ax.set_xlabel('Rate')
-        ax.set_yticks(range(n_rates))
-        ax.set_yticklabels(rate_keys)
-        ax.legend()
 
-        # Population size
-        ax = axs[1,0]
-        ax.plot(data.pop_years, data.pop_size, 'o', label='Data')
-        ax.plot(sim.pop_years,  sim.pop_size,  '-', label='Sim')
-        ax.set_title('Population size')
-        ax.set_xlabel('Year')
-        ax.set_ylabel('Population size')
-        ax.legend()
+            #%% Do the plotting!
 
-        # Population growth rate
-        ax = axs[2,0]
-        ax.plot(data.pop_years[:-1], data.pop_growth_rate, 'o', label='Data')
-        ax.plot(sim.pop_years[:-1],  sim.pop_growth_rate,  '-', label='Sim')
-        ax.set_title('Population growth rate')
-        ax.set_xlabel('Year')
-        ax.set_ylabel('Population growth rate')
-        ax.legend()
+            # Rates
+            ax = axs[0,0]
+            height = 0.4
+            n_rates = len(rate_keys)
+            y = np.arange(n_rates)
+            data_rates = np.array([data[k] for k in rate_keys])
+            sim_rates  = np.array([sim[k] for k in rate_keys])
+            ax.barh(y=y+height/2, width=data_rates, height=height, align='center', label='Data')
+            ax.barh(y=y-height/2, width=sim_rates,  height=height, align='center', label='Sim')
+            ax.set_title('Rates')
+            ax.set_xlabel('Rate')
+            ax.set_yticks(range(n_rates))
+            ax.set_yticklabels(rate_keys)
+            ax.legend()
 
-        # MCPR
-        ax = axs[3,0]
-        ax.plot(data.mcpr_years, data.mcpr, 'o', label='Data')
-        ax.plot(sim.mcpr_years,  sim.mcpr,  '-', label='Sim') # TODO: remove scale factor
-        ax.set_title('MCPR')
-        ax.set_xlabel('Year')
-        ax.set_ylabel('Modern contraceptive prevalence rate')
-        ax.legend()
+            # Population size
+            ax = axs[1,0]
+            ax.plot(data.pop_years, data.pop_size, 'o', label='Data')
+            ax.plot(sim.pop_years,  sim.pop_size,  '-', label='Sim')
+            ax.set_title('Population size')
+            ax.set_xlabel('Year')
+            ax.set_ylabel('Population size')
+            ax.legend()
 
-        # Data skyscraper
-        ax = axs[0,1]
-        ax.pcolormesh(self.age_bins, self.parity_bins, data.skyscrapers.transpose(), shading='nearest', cmap='turbo')
-        ax.set_aspect(1./ax.get_data_ratio()) # Make square
-        ax.set_title('Age-parity plot: data')
-        ax.set_xlabel('Age')
-        ax.set_ylabel('Parity')
+            # Population growth rate
+            ax = axs[2,0]
+            ax.plot(data.pop_years[:-1], data.pop_growth_rate, 'o', label='Data')
+            ax.plot(sim.pop_years[:-1],  sim.pop_growth_rate,  '-', label='Sim')
+            ax.set_title('Population growth rate')
+            ax.set_xlabel('Year')
+            ax.set_ylabel('Population growth rate')
+            ax.legend()
 
-        # Sim skyscraper
-        ax = axs[1,1]
-        ax.pcolormesh(self.age_bins, self.parity_bins, sim.skyscrapers.transpose(), shading='nearest', cmap='turbo')
-        ax.set_aspect(1./ax.get_data_ratio())
-        ax.set_title('Age-parity plot: sim')
-        ax.set_xlabel('Age')
-        ax.set_ylabel('Parity')
+            # MCPR
+            ax = axs[3,0]
+            ax.plot(data.mcpr_years, data.mcpr, 'o', label='Data')
+            ax.plot(sim.mcpr_years,  sim.mcpr,  '-', label='Sim')
+            ax.set_title('MCPR')
+            ax.set_xlabel('Year')
+            ax.set_ylabel('Modern contraceptive prevalence rate')
+            ax.legend()
 
-        # Spacing bins
+            # Data skyscraper
+            ax = axs[0,1]
+            ax.pcolormesh(self.age_bins, self.parity_bins, data.skyscrapers.transpose(), shading='nearest', cmap='turbo')
+            ax.set_aspect(1./ax.get_data_ratio()) # Make square
+            ax.set_title('Age-parity plot: data')
+            ax.set_xlabel('Age')
+            ax.set_ylabel('Parity')
 
-        ax = axs[2, 1]
-        height = 0.4
+            # Sim skyscraper
+            ax = axs[1,1]
+            ax.pcolormesh(self.age_bins, self.parity_bins, sim.skyscrapers.transpose(), shading='nearest', cmap='turbo')
+            ax.set_aspect(1./ax.get_data_ratio())
+            ax.set_title('Age-parity plot: sim')
+            ax.set_xlabel('Age')
+            ax.set_ylabel('Parity')
 
-        spacing_bins = sc.odict({'0-12': 0, '12-24': 1, '24-48': 2, '>48': 4})  # Spacing bins in years
-        n_bins = len(spacing_bins.keys())
+            # Spacing bins
+            ax = axs[2, 1]
+            height = 0.4
 
-        y = np.arange(len(data.spacing_bins))
-        ax.barh(y=y+height/2, width=data.spacing_bins, height=height, align='center', label='Data')
-        ax.barh(y=y-height/2, width=sim.spacing_bins,  height=height, align='center', label='Sim')
-        ax.set_title('Birth spacing bins')
-        ax.set_xlabel('Percent of births in each bin')
-        ax.set_yticks(range(n_bins))
-        ax.set_yticklabels(spacing_bins.keys())
-        ax.set_ylabel('Birth space in months')
-        ax.legend()
+            spacing_bins = sc.odict({'0-12': 0, '12-24': 1, '24-48': 2, '>48': 4})  # Spacing bins in years
+            n_bins = len(spacing_bins.keys())
 
-        # Age first stats
+            y = np.arange(len(data.spacing_bins))
+            ax.barh(y=y+height/2, width=data.spacing_bins, height=height, align='center', label='Data')
+            ax.barh(y=y-height/2, width=sim.spacing_bins,  height=height, align='center', label='Sim')
+            ax.set_title('Birth spacing bins')
+            ax.set_xlabel('Percent of births in each bin')
+            ax.set_yticks(range(n_bins))
+            ax.set_yticklabels(spacing_bins.keys())
+            ax.set_ylabel('Birth space in months')
+            ax.legend()
 
-        quartile_keys = ['25th %',
-                     'Median',
-                     '75th %']
-        n_quartiles = len(quartile_keys)
+            # Age first stats
+            quartile_keys = ['25th %',
+                         'Median',
+                         '75th %']
+            n_quartiles = len(quartile_keys)
 
-        ax = axs[3,1]
-        height = 0.4
-        y = np.arange(len(data.age_first_stats))
-        ax.barh(y=y+height/2, width=data.age_first_stats, height=height, align='center', label='Data')
-        ax.barh(y=y-height/2, width=sim.age_first_stats,  height=height, align='center', label='Sim')
-        ax.set_title('Age at first birth')
-        ax.set_xlabel('Age')
-        ax.set_yticks(range(n_quartiles))
-        ax.set_yticklabels(quartile_keys)
-        ax.legend()
+            ax = axs[3,1]
+            height = 0.4
+            y = np.arange(len(data.age_first_stats))
+            ax.barh(y=y+height/2, width=data.age_first_stats, height=height, align='center', label='Data')
+            ax.barh(y=y-height/2, width=sim.age_first_stats,  height=height, align='center', label='Sim')
+            ax.set_title('Age at first birth')
+            ax.set_xlabel('Age')
+            ax.set_yticks(range(n_quartiles))
+            ax.set_yticklabels(quartile_keys)
+            ax.legend()
 
-        # Age pregnant stats
-        ax = axs[0,2]
-        height = 0.4
-        y = np.arange(len(data.age_pregnant_stats))
-        ax.barh(y=y+height/2, width=data.age_pregnant_stats, height=height, align='center', label='Data')
-        ax.barh(y=y-height/2, width=sim.age_pregnant_stats,  height=height, align='center', label='Sim')
-        ax.set_title('Age of women currently pregnant')
-        ax.set_xlabel('Age')
-        ax.set_yticks(range(n_quartiles))
-        ax.set_yticklabels(quartile_keys)
-        ax.legend()
+            # Age pregnant stats
+            ax = axs[0,2]
+            height = 0.4
+            y = np.arange(len(data.age_pregnant_stats))
+            ax.barh(y=y+height/2, width=data.age_pregnant_stats, height=height, align='center', label='Data')
+            ax.barh(y=y-height/2, width=sim.age_pregnant_stats,  height=height, align='center', label='Sim')
+            ax.set_title('Age of women currently pregnant')
+            ax.set_xlabel('Age')
+            ax.set_yticks(range(n_quartiles))
+            ax.set_yticklabels(quartile_keys)
+            ax.legend()
 
-        # Age parity stats
-        ax = axs[1,2]
-        cols = sc.gridcolors(3)
-        for i,yvals in enumerate([data.age_parity_stats, sim.age_parity_stats]):
-            for j in range(3):
-                vals = yvals[:,j]
-                if i==0:
-                    marker = 'o'
-                    label = 'Data'
-                else:
-                    marker = '-'
-                    label = 'Sim'
-                ax.plot(vals, marker, c=cols[j], label=label)
-        ax.set_title('Age parity stats - quartiles')
-        ax.set_xlabel('Parity')
-        ax.set_ylabel('Age')
-        ax.legend()
+            # Age parity stats
+            ax = axs[1,2]
+            cols = sc.gridcolors(3)
+            for i,yvals in enumerate([data.age_parity_stats, sim.age_parity_stats]):
+                for j in range(3):
+                    vals = yvals[:,j]
+                    if i==0:
+                        marker = 'o'
+                        label = 'Data'
+                    else:
+                        marker = '-'
+                        label = 'Sim'
+                    ax.plot(vals, marker, c=cols[j], label=label)
+            ax.set_title('Age parity stats - quartiles')
+            ax.set_xlabel('Parity')
+            ax.set_ylabel('Age')
+            ax.legend()
 
-        # Method counts
-        ax = axs[2,2]
+            # Method counts
+            ax = axs[2,2]
 
-        method_labels = ['None',
-                    'Pill',
-                    'IUDs',
-                    'Injectables',
-                    'Condoms',
-                    'BTL',
-                    'Rhythm',
-                    'Withdrawal',
-                    'Implants',
-                    'Other']
+            height = 0.4
+            y = np.arange(len(data.method_counts))
+            y1 = y + height/2
+            y2 = y - height/2
+            ax.barh(y=y1, width=data.method_counts, height=height, align='center', label='Data')
+            ax.barh(y=y2, width=sim.method_counts,  height=height, align='center', label='Sim')
+            ax.set_yticks(y, self.method_keys)
+            ax.set_title('Method counts')
+            ax.set_ylabel('Contraceptive method')
+            ax.set_xlabel('Rate of use')
+            ax.legend()
 
-        ax.pie(data.method_counts[:], labels=method_labels)
-        ax.set_title('Method use in data')
-
-        #width = 0.4
-        #x = np.arange(len(data.method_counts))
-        #ax.bar(x=x+width/2, height=data.method_counts, width=width, align='center', label='Data')
-        #ax.bar(x=x-width/2, height=sim.method_counts,  width=width, align='center', label='Sim')
-        #ax.set_title('Method counts')
-        #ax.set_xlabel('Contraceptive method')
-        #ax.set_ylabel('Rate of use')
-        #ax.legend()
-
-        ax = axs[3,2]
-
-        ax.pie(sim.method_counts[:], labels=method_labels)
-        ax.set_title('Method use in model')
+            # ASFR
+            ax = axs[3,2]
+            y = np.arange(len(data.asfr))
+            y1 = y + height/2
+            y2 = y - height/2
+            ax.barh(y=y1, width=data.asfr, height=height, align='center', label='Data')
+            ax.barh(y=y2, width=sim.asfr,  height=height, align='center', label='Sim')
+            ax.set_yticks(y, sim.asfr_bins)
+            ax.set_title('Age-specific fertility rate')
+            ax.set_ylabel('Age bin')
+            ax.set_xlabel('Fertility rate')
+            ax.legend()
 
         # Tidy up
         if do_maximize:
             sc.maximize(fig=fig)
 
-        if do_show:
-            pl.show()
-
-        pass
+        return fps.tidy_up(fig=fig, do_show=do_show, do_save=do_save, filename=filename)
 
 
 
@@ -829,10 +851,10 @@ class Fit(sc.prettyobj):
 
         # Remove keys that aren't for fitting
         for key in self.data.keys():
-            if key.endswith('_years'):
+            if key.endswith('_years') or key.endswith('_bins'):
                 self.data.pop(key)
                 self.sim_results.pop(key)
-        self.keys       = data.keys()
+        self.keys = data.keys()
 
         # These are populated during initialization
         self.inds         = sc.objdict() # To store matching indices between the data and the simulation
@@ -866,7 +888,7 @@ class Fit(sc.prettyobj):
 
         data_cols = set(self.data.keys())
 
-        if self.keys is None:
+        if self.keys is None: # pragma: nocover
             sim_keys = self.sim_results.keys()
             intersection = list(set(sim_keys).intersection(data_cols)) # Find keys in both the sim and data
             self.keys = [key for key in sim_keys if key in intersection and key.startswith('cum_')] # Only keep cumulative keys
@@ -874,7 +896,7 @@ class Fit(sc.prettyobj):
                 errormsg = f'No matches found between simulation result keys ({sim_keys}) and data columns ({data_cols})'
                 raise sc.KeyNotFoundError(errormsg)
         mismatches = [key for key in self.keys if key not in data_cols]
-        if len(mismatches):
+        if len(mismatches): # pragma: nocover
             mismatchstr = ', '.join(mismatches)
             errormsg = f'The following requested key(s) were not found in the data: {mismatchstr}'
             raise sc.KeyNotFoundError(errormsg)
@@ -909,7 +931,7 @@ class Fit(sc.prettyobj):
 
         # Process custom inputs
         self.custom_keys = list(self.custom.keys())
-        for key in self.custom.keys():
+        for key in self.custom.keys(): # pragma: nocover
 
             # Initialize and do error checking
             custom = self.custom[key]
@@ -1098,7 +1120,7 @@ def compute_gof(actual, predicted, normalize=True, use_frac=False, use_squared=F
     predicted = np.array(sc.dcp(predicted), dtype=float)
 
     # Custom estimator is supplied: use that
-    if skestimator is not None:
+    if skestimator is not None: # pragma: nocover
         try:
             import sklearn.metrics as sm
             sklearn_gof = getattr(sm, skestimator) # Shortcut to e.g. sklearn.metrics.max_error
@@ -1194,7 +1216,7 @@ def diff_summaries(sim1, sim2, skip_key_diffs=False, output=False, die=False):
             if sim1_val != sim2_val and not both_nan:
                 mismatches[key] = {'sim1': sim1_val, 'sim2': sim2_val}
 
-    if len(mismatches):
+    if len(mismatches): # pragma: nocover
         valmatchmsg = '\nThe following values differ between the two simulations:\n'
         df = pd.DataFrame.from_dict(mismatches).transpose()
         diff   = []
