@@ -3,12 +3,9 @@ Specify the core interventions available in FPsim. Other interventions can be
 defined by the user by inheriting from these classes.
 '''
 
-import numpy as np
 import pylab as pl
 import sciris as sc
 import inspect
-from . import utils as fpu
-from . import defaults as fpd
 
 
 #%% Generic intervention classes
@@ -37,7 +34,7 @@ class Intervention:
         self.show_label = show_label # Do not show the label by default
         self.do_plot = do_plot if do_plot is not None else True # Plot the intervention, including if None
         self.line_args = sc.mergedicts(dict(linestyle='--', c='#aaa', lw=1.0), line_args) # Do not set alpha by default due to the issue of overlapping interventions
-        self.days = [] # The start and end days of the intervention
+        self.years = [] # The start and end years of the intervention
         self.initialized = False # Whether or not it has been initialized
         self.finalized = False # Whether or not it has been initialized
         return
@@ -146,20 +143,24 @@ class Intervention:
         if self.do_plot or self.do_plot is None:
             if ax is None:
                 ax = pl.gca()
-            if hasattr(self, 'plot_days'):
-                days = self.plot_days
+
+            if hasattr(self, 'plot_years'):
+                years = self.plot_years
+            elif not self.years and hasattr(self, 'year'):
+                years = sc.toarray(self.year)
             else:
-                days = self.days
-            if sc.isiterable(days):
+                years = self.years
+
+            if sc.isiterable(years):
                 label_shown = False # Don't show the label more than once
-                for day in days:
-                    if sc.isnumber(day):
+                for y in years:
+                    if sc.isnumber(y):
                         if self.show_label and not label_shown: # Choose whether to include the label in the legend
                             label = self.label
                             label_shown = True
                         else:
                             label = None
-                        ax.axvline(day, label=label, **line_args)
+                        ax.axvline(y, label=label, **line_args)
         return
 
 
@@ -285,7 +286,7 @@ class change_par(Intervention):
         return
 
 
-    def finalize(self):
+    def finalize(self, sim=None):
         # Check that all changes were applied
         n_counter = self.counter
         n_vals = len(self.vals)
@@ -295,28 +296,6 @@ class change_par(Intervention):
         return
 
 
-def key2ind(sim, key):
-    """
-    Take a method key and convert to an int, e.g. 'Condoms' → 7
-    """
-    ind = key
-    if ind in fpd.none_all_keys:
-        ind = slice(None) # This is equivalent to ":" in matrix[:,:]
-    elif isinstance(ind, str):
-        ind = sim.pars['methods']['map'][key]
-    return ind
-
-
-def getval(v):
-    ''' Handle different ways of supplying a value -- number, distribution, function '''
-    if sc.isnumber(v):
-        return v
-    elif isinstance(v, dict):
-        return fpu.sample(**v)[0]
-    elif callable(v):
-        return v()
-
-
 class update_methods(Intervention):
     """
     Intervention to modify method efficacy and/or switching matrix.
@@ -333,12 +312,12 @@ class update_methods(Intervention):
 
         probs (list): A list of dictionaries where each dictionary has the following keys:
 
-            source (str): The source method to be changed.
-            dest (str) The destination method to be changed.
-            factor (float): The factor by which to multiply existing probability; OR
-            value (float): The value to replace the switching probability value.
-            keys (list): A list of strings representing age groups to affect.
-            matrix (str): One of ['probs', 'probs1', 'probs1to6'] where:
+            source (str): the source method to be changed.
+            dest   (str): the destination method to be changed.
+            factor (float): the factor by which to multiply existing probability; OR
+            value  (float): the value to replace the switching probability value.
+            keys   (list): a list of strings representing age groups to affect.
+            matrix (str): one of ['probs', 'probs1', 'probs1to6'] where:
 
                 probs:     Changes the specified uptake at the corresponding year regardless of state.
                 probs1:    Changes the specified uptake for all individuals in their first month postpartum.
@@ -378,39 +357,29 @@ class update_methods(Intervention):
             # Implement efficacy
             if self.eff is not None:
                 for k,rawval in self.eff.items():
-                    try:
-                        key2ind(sim, k)
-                    except:
-                        errormsg = f'Key "{k}" is not a valid method: are you sure this is an efficacy change?'
-                        raise ValueError(errormsg)
-                    v = getval(rawval)
-                    ind = key2ind(sim, k)
-                    orig = sim['method_efficacy'][ind]
-                    sim['method_efficacy'][ind] = v
-                    if self.verbose:
-                        print(f'At time {sim.y:0.1f}, efficacy for method {k} was changed from {orig:0.3f} to {v:0.3f}')
+                    sim.pars.update_method_eff(method=k, eff=rawval)
 
             # Implement method mix shift
-            raw = sim['methods']['raw'] # We adjust the raw matrices, so the effects are persistent
             if self.probs is not None:
                 probs = sc.tolist(self.probs)
                 for entry in probs:
                     entry = sc.dcp(entry)
-                    s_matrix = entry.pop('matrix', self.matrix) # Switching matrix
-                    ages     = entry.pop('ages', None)
-                    source   = entry.pop('source', None)
-                    dest     = entry.pop('dest', None)
-                    method   = entry.pop('method', None)
-                    factor   = entry.pop('factor', None)
-                    value    = entry.pop('value', None)
-                    i_factor = entry.pop('init_factor', None)
-                    d_factor = entry.pop('discont_factor', None)
-                    i_value  = entry.pop('init_value', None)
-                    d_value  = entry.pop('discont_value', None)
+                    matrix    = entry.pop('matrix', self.matrix) # Switching matrix
+                    ages      = entry.pop('ages', None)
+                    source    = entry.pop('source', None)
+                    dest      = entry.pop('dest', None)
+                    method    = entry.pop('method', None)
+                    factor    = entry.pop('factor', None)
+                    value     = entry.pop('value', None)
+                    i_factor  = entry.pop('init_factor', None)
+                    d_factor  = entry.pop('discont_factor', None)
+                    i_value   = entry.pop('init_value', None)
+                    d_value   = entry.pop('discont_value', None)
+                    copy_from = entry.pop('copy_from', None)
 
                     # Supply default matrix
-                    if s_matrix is None:
-                        s_matrix = 'annual'
+                    if matrix is None:
+                        matrix = 'annual'
 
                     # Validation # CK: TODO: move validation to initialization
                     if len(entry) != 0:
@@ -427,16 +396,17 @@ class update_methods(Intervention):
                             dest = method
 
                     # Ensure correct number of inputs are given
-                    n_vals = len(sc.mergelists(factor, value, i_factor, d_factor, i_value, d_value))
+                    n_vals = len(sc.mergelists(copy_from, factor, value, i_factor, d_factor, i_value, d_value))
                     if n_vals != 1:
-                        errormsg = f'Must supply one and only one of factor, value, or initiation/discontinuation factors/values; you supplied {n_vals}'
+                        errormsg = f'Must supply one and only one of copy_from, factor, value, or initiation/discontinuation factors/values; you supplied {n_vals}'
                         raise ValueError(errormsg)
 
                     # Check nothing strange has happened
                     is_switch  = len(sc.mergelists(factor, value))
                     is_init    = len(sc.mergelists(i_value, i_factor))
                     is_discont = len(sc.mergelists(d_value, d_factor))
-                    if is_switch + is_init + is_discont != 1:
+                    is_copy    = (copy_from is not None)
+                    if is_switch + is_init + is_discont + is_copy != 1:
                         errormsg = f'Could not figure out what to do: switching={is_switch}, initiation={is_init}, discontinuation={is_discont}, but only one should happen'
                         raise ValueError(errormsg)
 
@@ -446,52 +416,18 @@ class update_methods(Intervention):
                     elif is_discont: # It's discontinuation
                         source = method
                         dest = 'None'
-                    elif (source is None) and (dest is None):
+                    elif not is_copy and (source is None) and (dest is None):
                         errormsg = 'Must supply a source or a destination'
                         raise ValueError(errormsg)
 
-                    # Convert from strings to indices
-                    source = key2ind(sim, source)
-                    dest = key2ind(sim, dest)
-
                     # Decide if it's a factor or a value modification
                     factor = sc.mergelists(factor, i_factor, d_factor)
                     value  = sc.mergelists(value, i_value, d_value)
                     factor = factor[0] if factor else None
                     value  = value[0]  if value  else None
 
-                    # Replace age keys with all ages if so asked
-                    if ages in fpd.none_all_keys:
-                        ages = raw['annual'].keys()
-                    else:
-                        ages = sc.tolist(ages)
-
-                    # Actually loop over the matrices and apply the changes
-                    for k in ages:
-                        matrix = raw[s_matrix][k]
-                        if s_matrix == 'pp0to1': # Handle the postpartum initialization *vector*
-                           orig = matrix[dest]
-                           if factor is not None:
-                               matrix[dest] *= getval(factor)
-                           elif value is not None:
-                               val = getval(value)
-                               matrix[dest] = 0
-                               matrix *= (1-val)/matrix.sum()
-                               matrix[dest] = val
-                               assert np.isclose(matrix.sum(), 1, atol=1e-3), f'Matrix should sum to 1, not {matrix.sum()}'
-                           if self.verbose:
-                               print(f'At time {sim.y:0.1f}, matrix {s_matrix} for age group {k} was changed from:\n{orig}\nto\n{matrix[dest]}')
-                        else: # Handle annual switching *matrices*
-                            orig = matrix[source, dest]
-                            if factor is not None:
-                                matrix[source, dest] *= getval(factor)
-                            elif value is not None:
-                                val = getval(value)
-                                matrix[source, dest] = 0
-                                matrix[source, :] *= (1-val)/matrix[source, :].sum()
-                                matrix[source, dest] = val
-                                assert np.isclose(matrix[source, :].sum(), 1, atol=1e-3), f'Matrix should sum to 1, not {matrix.sum()}'
-                            if self.verbose:
-                                print(f'At time {sim.y:0.1f}, matrix {self.matrix} for age group {k} was changed from:\n{orig}\nto\n{matrix[source, dest]}')
+                    # Actually update the values and check the matrix is valid
+                    kw = dict(source=source, dest=dest, factor=factor, value=value, ages=ages, matrix=matrix, copy_from=copy_from)
+                    sim.pars.update_method_prob(**kw)
 
         return
\ No newline at end of file
