@@ -6,6 +6,7 @@ import numpy as np
 import pandas as pd
 import sciris as sc
 from . import defaults as fpd
+from . import parameters as fpp
 from . import sim as fps
 from . import interventions as fpi
 
@@ -18,7 +19,7 @@ def check_not_none(obj, *args):
     ''' Check that all needed inputs are supplied '''
     for key in args:
         if obj[key] is None:
-            errormsg = f'Entry "{key}" is not allowed to be None for scenario spec "{obj.which}"'
+            errormsg = f'Entry "{key}" is not allowed to be None for the following scenario spec:\n{obj}'
             raise ValueError(errormsg)
     return
 
@@ -58,6 +59,7 @@ class Scenario(sc.prettyobj, sc.dictobj):
         spec   (dict): a pre-made specification of a scenario; see keyword explanations below (optional)
         args   (list): additional specifications (optional)
         label  (str): the sim label to use for this scenario
+        pars   (dict): optionally supply additional sim parameters to use with this scenario (that take effect at the beginning of the sim, not at the point of intervention)
         year   (float): the year at which to activate efficacy and probability scenarios
         matrix (str): which set of probabilities to modify for probability scenarios (e.g. annual or postpartum)
         ages   (str/list): the age groups to modify the probabilities for
@@ -74,6 +76,7 @@ class Scenario(sc.prettyobj, sc.dictobj):
         dest   (str): the method to switch to
         factor (float): if supplied, multiply the [source, dest] probability by this amount
         value  (float): if supplied, instead of factor, replace the [source, dest] probability by this value
+        copy_from (str): if supplied, copy probabilities from a different method
 
     Args (initiation/discontinuation):
         year   (float): as above
@@ -87,8 +90,8 @@ class Scenario(sc.prettyobj, sc.dictobj):
 
     Args (parameter):
         par (str): the parameter to modify
-        years (float/list): the year(s) at which to apply the modifications
-        vals (float/list): the value(s) of the parameter for each year
+        par_years (float/list): the year(s) at which to apply the modifications
+        par_vals (float/list): the value(s) of the parameter for each year
 
     Args (custom):
         interventions (Intervention/list): any custom intervention(s) to be applied to the scenario
@@ -116,26 +119,32 @@ class Scenario(sc.prettyobj, sc.dictobj):
         def update_sim(sim): sim.updated = True
         s6 = fp.make_scen(interventions=update_sim)
 
-        # Combining multiple scenarios: increase injectables initiation and reduce exposure factor
+        # Combining multiple scenarios: change probabilities and exposure factor
         s7 = fp.make_scen(
-            dict(method='Injectables', init_factor=2),
+            dict(method='Injectables', init_value=0.1, discont_value=0.02, create=True),
             dict(par='exposure_factor', years=2010, vals=0.5)
         )
 
         # Scenarios can be combined
         s8 = s1 + s2
     '''
-    def __init__(self, spec=None, *args, label=None, year=None, matrix=None, ages=None, # Basic settings
+    def __init__(self, spec=None, label=None, pars=None, year=None, matrix=None, ages=None, # Basic settings
                  eff=None, probs=None, # Option 1
-                 source=None, dest=None, factor=None, value=None, # Option 2
+                 source=None, dest=None, factor=None, value=None, copy_from=None, # Option 2
                  method=None, init_factor=None, discont_factor=None, init_value=None, discont_value=None, # Option 3
-                 par=None, years=None, vals=None, # Option 4
+                 par=None, par_years=None, par_vals=None, # Option 4
                  interventions=None, # Option 5
                  ):
 
         # Handle input specification
-        self.specs = sc.mergelists(*[Scenario(**spec).specs for spec in sc.mergelists(spec, *args)]) # Sorry
+        if isinstance(spec, str) and not isinstance(label, str): # Swap order if types don't match
+            label,spec = spec,label
+        self.specs = sc.mergelists(*[Scenario(**s).specs for s in sc.tolist(spec)]) # Sorry
         self.label = label
+        self.pars  = sc.mergedicts(pars)
+        if not isinstance(label, (str, type(None))):
+            errormsg = f'Unexpected label type {type(label)}'
+            raise TypeError(errormsg)
 
         # Handle other keyword inputs
         eff_spec   = None
@@ -170,6 +179,7 @@ class Scenario(sc.prettyobj, sc.dictobj):
                     discont_factor = discont_factor,
                     init_value     = init_value,
                     discont_value  = discont_value,
+                    copy_from      = copy_from,
                 )
             )
             check_not_none(prob_spec, 'year')
@@ -181,10 +191,10 @@ class Scenario(sc.prettyobj, sc.dictobj):
             par_spec = sc.objdict(
                 which = 'par',
                 par   = par,
-                years = years,
-                vals  = vals,
+                par_years = par_years,
+                par_vals  = par_vals,
             )
-            check_not_none(par_spec, 'years', 'vals')
+            check_not_none(par_spec, 'par_years', 'par_vals')
 
         # It's a custom scenario(s)
         if interventions is not None:
@@ -219,15 +229,38 @@ class Scenario(sc.prettyobj, sc.dictobj):
 
     def update_label(self, label=None):
         ''' Ensure all specs have the correct label '''
+
+        # Option 1: update the label based on the spec
         if label is None:
             label = self.label
-        if label is not None:
+            for spec in self.specs:
+                if 'label' in spec and spec['label'] is not None:
+                    label = spec['label']
+
+        # Option 2: update the spec based on the label
+        else:
             for spec in self.specs:
                 spec['label'] = label
+
         self.label = label
         return
 
 
+    def run(self, run_args=None, **kwargs):
+        '''
+        Shortcut for creating and running a Scenarios object based on the current scenario.
+
+        Args:
+            run_args (dict): passed to scens.run()
+            kwargs (dict): passed to Scenarios()
+
+        '''
+        scens = Scenarios(**kwargs)
+        scens.add_scen(self)
+        scens.run(**sc.mergedicts(run_args))
+        return scens
+
+
 
 def make_scen(*args, **kwargs):
     ''' Alias for ``fp.Scenario()``. '''
@@ -278,7 +311,7 @@ class Scenarios(sc.prettyobj):
             scen.update_label(label)
             self.scens.append(scen)
         else:
-            self.scens.append(Scenario(scen, label=label))
+            self.scens.append(Scenario(label=label, spec=scen))
         return
 
 
@@ -289,7 +322,7 @@ class Scenarios(sc.prettyobj):
             raise ValueError(errormsg)
         sims = sc.autolist()
         for i in range(self.repeats):
-            pars = sc.mergedicts(fpd.pars(self.pars.get('location')), self.pars, _copy=True)
+            pars = sc.mergedicts(fpp.pars(self.pars.get('location')), self.pars, _copy=True)
             pars.update(kwargs)
             pars['seed'] += i
             sim = fps.Sim(pars=pars)
@@ -303,7 +336,7 @@ class Scenarios(sc.prettyobj):
     def make_scens(self):
         ''' Convert a scenario specification into a list of sims '''
         for i,scen in enumerate(self.scens):
-            simlabel = None
+            simlabel = scen.label
             interventions = sc.autolist()
             for spec in sc.tolist(scen.specs):
 
@@ -315,26 +348,29 @@ class Scenarios(sc.prettyobj):
                 else:
                     which = spec.pop('which')
 
-                # Handle update_methods
+                # Handle interventions
                 if which == 'eff':
-                    eff = spec.pop('eff')
+                    eff  = spec.pop('eff')
                     year = spec.pop('year')
                     interventions += fpi.update_methods(eff=eff, year=year)
                 elif which == 'prob':
                     probs = spec.pop('probs')
-                    year = spec.pop('year')
+                    year  = spec.pop('year')
                     interventions += fpi.update_methods(probs=probs, year=year)
                 elif which == 'par':
                     par = spec.pop('par')
-                    years = spec.pop('years')
-                    vals = spec.pop('vals')
+                    years = spec.pop('par_years')
+                    vals  = spec.pop('par_vals')
                     interventions += fpi.change_par(par=par, years=years, vals=vals)
                 elif which == 'intv':
                     intv = spec.pop('intervention')
                     interventions += intv
+                else:
+                    errormsg = f'Could not understand intervention type "{which}"'
+                    raise ValueError(errormsg)
 
                 # Handle label
-                label  = spec.pop('label', None)
+                label  = spec.pop('label', scen.label)
                 assert len(spec)==0, f'Unrecognized scenario key(s) {sc.strjoin(spec.keys())}'
                 if simlabel is None:
                     simlabel = label
@@ -344,12 +380,12 @@ class Scenarios(sc.prettyobj):
 
             if simlabel is None:
                 simlabel = f'Scenario {i}'
-            sims = self.make_sims(interventions=interventions, scenlabel=simlabel)
+            sims = self.make_sims(scenlabel=simlabel, interventions=interventions, **scen.pars)
             self.simslist.append(sims)
         return
 
 
-    def run(self, *args, **kwargs):
+    def run(self, recompute=True, *args, **kwargs):
         ''' Actually run a list of sims '''
 
         # Check that it's set up
@@ -370,10 +406,11 @@ class Scenarios(sc.prettyobj):
         self.already_run = True
 
         # Process
-        self.msim_merged =self.msim.remerge()
+        self.msim_merged =self.msim.remerge(recompute=recompute)
         self.analyze_sims()
         return
 
+
     def check_run(self):
         ''' Give a meaningful error message if the scenarios haven't been run '''
         if not self.already_run:
@@ -382,26 +419,22 @@ class Scenarios(sc.prettyobj):
         return
 
 
-    def plot_sims(self, **kwargs):
-        ''' Plot each sim as a separate line across all senarios '''
+    def plot(self, to_plot=None, plot_sims=True, **kwargs):
+        ''' Plot the scenarios with bands -- see ``sim.plot()`` for args '''
         self.check_run()
-        return self.msim.plot(plot_sims=True, **kwargs)
+        if to_plot == 'method':
+            return self.msim.plot(to_plot=to_plot, plot_sims=plot_sims, **kwargs)
+        else:
+            return self.msim_merged.plot(to_plot=to_plot, plot_sims=plot_sims, **kwargs)
 
 
-    def plot_scens(self, **kwargs):
-        ''' Plot the scenarios with bands '''
+    def plot_sims(self, to_plot=None, plot_sims=True, **kwargs):
+        ''' Plot each sim as a separate line across all senarios -- see ``sim.plot()`` for args '''
         self.check_run()
-        return self.msim_merged.plot(plot_sims=True, **kwargs)
-
-
-    def plot_cpr(self, **kwargs):
-        ''' Plot the CPR with bands '''
-        self.check_run()
-        return self.msim_merged.plot_cpr(**kwargs)
-
+        return self.msim.plot(to_plot=to_plot, plot_sims=plot_sims, **kwargs)
 
     def analyze_sims(self, start=None, end=None):
-        ''' Take a list of sims that have different labels and count the births in each '''
+        ''' Take a list of sims that have different labels and extrapolate statistics from each '''
         self.check_run()
 
         # Pull out first sim and parameters
@@ -409,33 +442,31 @@ class Scenarios(sc.prettyobj):
         if start is None: start = sim0.pars['start_year']
         if end   is None: end   = sim0.pars['end_year']
 
-        def count_births(sim):
-            year = sim.results['t']
-            births = sim.results['births']
-            inds = sc.findinds((year >= start), year < end)
-            output = births[inds].sum()
-            return output
+        def analyze_sim(sim):
+            def aggregate_channel(channel, is_sum=True, is_t=False):
+                if is_t:
+                    year = sim.results['t']
+                else:
+                    year = sim.results['tfr_years']
+                channel_results = sim.results[channel]
+                inds = sc.findinds((year >= start), year <= end)
 
-        def method_failure(sim):
-            year = sim.results['tfr_years']
-            meth_fail = sim.results['method_failures_over_year']
-            inds = sc.findinds((year >= start), year < end)
-            output = meth_fail[inds].sum()
-            return output
+                if is_sum:
+                    return channel_results[inds].sum()
+                else:
+                    return channel_results[inds].mean()
 
-        def count_pop(sim):
-            year = sim.results['tfr_years']
-            popsize = sim.results['pop_size']
-            inds = sc.findinds((year >= start), year < end)
-            output = popsize[inds].sum()
-            return output
+            # Defines how we calculate each channel, first number is is_sum: 1 = aggregate as sum, 0 = aggregate as mean
+            # The second parameter defines whether to aggregate by year or by timestep where 1 = use sim.t (timestep), 0 = use sim.tfr_years (years)
+            agg_param_dict = {'method_failures_over_year': (1, 0), 'pop_size': (1, 0), 'tfr_rates': (0, 0), 'maternal_deaths_over_year': (1, 0),
+                            'infant_deaths_over_year': (1, 0), 'mcpr': (0, 1), 'births': (1, 1)}
+            results_dict = {}
 
-        def mean_tfr(sim):
-            year = sim.results['tfr_years']
-            rates = sim.results['tfr_rates']
-            inds = sc.findinds((year >= start), year < end)
-            output = rates[inds].mean()
-            return output
+            for channel in agg_param_dict:
+                parameters = agg_param_dict[channel]
+                results_dict[channel] = aggregate_channel(channel, parameters[0], parameters[1])
+
+            return results_dict
 
         # Split the sims up by scenario
         results = sc.objdict()
@@ -453,15 +484,16 @@ class Scenarios(sc.prettyobj):
         raw = sc.ddict(list)
         for key,sims in results.sims.items():
             for sim in sims:
-                n_births = count_births(sim)
-                n_fails  = method_failure(sim)
-                n_pop = count_pop(sim)
-                n_tfr = mean_tfr(sim)
+                results_dict = analyze_sim(sim)
                 raw['scenario'] += [key]      # Append scenario key
-                raw['births']   += [n_births] # Append births
-                raw['fails']    += [n_fails]  # Append failures
-                raw['popsize']  += [n_pop]    # Append population size
-                raw['tfr']      += [n_tfr]    # Append mean tfr rates
+                raw['births']   += [results_dict['births']] # Append births
+                raw['fails']    += [results_dict['method_failures_over_year']]  # Append failures
+                raw['popsize']  += [results_dict['pop_size']]    # Append population size
+                raw['tfr']      += [results_dict['tfr_rates']]    # Append mean tfr rates
+                raw['infant_deaths'] += [results_dict['infant_deaths_over_year']]
+                raw['maternal_deaths'] += [results_dict['maternal_deaths_over_year']]
+                raw['mcpr'] += [results_dict['mcpr']]
+
 
         # Calculate basic stats
         results.stats = sc.objdict()
@@ -475,4 +507,4 @@ class Scenarios(sc.prettyobj):
         results.df = pd.DataFrame(raw)
         self.results = results
 
-        return
\ No newline at end of file
+        return
